@@ -29,13 +29,19 @@ calc.2LL = function(fit, nnodes.gq=8, nsd.gq=4) {
         ## .env$model$assignPtr()
         .evtM  <- saem.cfg$evtM
         .rx <- .env$model
+        RxODE::rxLoad(.env$model)
         .pars <- .rx$params
         .pars <- setNames(rep(1.1,length(.pars)),.pars);
         suppressWarnings(do.call(RxODE:::rxSolve.default,
                                 c(list(object=.rx, params=.pars,
-                                       events=.evtM,.setupOnly=2L),
+                                       events=.evtM,.setupOnly=1L),
                                   saem.cfg$optM)));
+        RxODE::rxDynProtect(RxODE::rxDll(.rx))
+        on.exit({RxODE::rxDynProtect("")})
     }
+    dyn.load(.env$saem.dll);
+    assignInMyNamespace(".protectSaemDll", .env$saem.dll)
+    on.exit({assignInMyNamespace(".protectSaemDll", "")}, add=TRUE)
     dopred = attr(fit, "dopred")
     resMat = fit$resMat
     ares = resMat[,1]
@@ -133,8 +139,10 @@ plot.saemFit = function(x,...) {
         .pars <- setNames(rep(1.1,length(.pars)),.pars);
         suppressWarnings(do.call(RxODE:::rxSolve.default,
                                 c(list(object=.rx, params=.pars,
-                                       events=.evtM,.setupOnly=2L),
+                                       events=.evtM,.setupOnly=1L),
                                   saem.cfg$optM)));
+        RxODE::rxDynProtect(RxODE::rxDll(.rx))
+        on.exit({RxODE::rxDynProtect("")})
     }
     dat = as.data.frame(saem.cfg$evt)
     dat = cbind(dat[dat$EVID == 0, ], DV = saem.cfg$y)
@@ -318,16 +326,22 @@ calc.COV = function(fit0) {
   .env <- attr(fit,"env");
   saem.cfg  <-  attr(fit, "saem.cfg")
   if (.env$is.ode){
-    ## .env$model$assignPtr()
-    .evtM  <- saem.cfg$evtM
-    .rx <- .env$model
-    .pars <- .rx$params
-    .pars <- setNames(rep(1.1,length(.pars)),.pars);
-    suppressWarnings(do.call(RxODE:::rxSolve.default,
-                             c(list(object=.rx, params=.pars,
-                                    events=.evtM,.setupOnly=2L),
-                                    saem.cfg$optM)));
+      ## .env$model$assignPtr()
+      .evtM  <- saem.cfg$evtM
+      .rx <- .env$model
+      RxODE::rxLoad(.rx)
+      .pars <- .rx$params
+      .pars <- setNames(rep(1.1,length(.pars)),.pars);
+      suppressWarnings(do.call(RxODE:::rxSolve.default,
+                               c(list(object=.rx, params=.pars,
+                                      events=.evtM,.setupOnly=1L),
+                                 saem.cfg$optM)));
+      RxODE::rxDynProtect(RxODE::rxDll(.rx))
+      on.exit({RxODE::rxDynProtect("")})
   }
+  dyn.load(.env$saem.dll);
+  assignInMyNamespace(".protectSaemDll", .env$saem.dll)
+  on.exit({assignInMyNamespace(".protectSaemDll", "")}, add=TRUE)
   dopred = attr(fit, "dopred")
   resMat = fit$resMat
   ares = resMat[,1]
