@@ -31,10 +31,15 @@ vpc_saemFit = function(fit, dat, nsim = 100, by=NULL, ...) {
       .pars <- .rx$params
       .pars <- setNames(rep(1.1,length(.pars)),.pars);
       suppressWarnings(do.call(RxODE:::rxSolve.default,
-                              c(list(object=.rx, params=.pars,
-                                     events=.evtM,.setupOnly=2L),
-                                saem.cfg$optM)));
+                               c(list(object=.rx, params=.pars,
+                                      events=.evtM,.setupOnly=2L),
+                                 saem.cfg$optM)));
+      RxODE::rxDynProtect(RxODE::rxDll(.rx))
+      on.exit({RxODE::rxDynProtect("")})
   }
+  dyn.load(.env$saem.dll);
+  assignInMyNamespace(".protectSaemDll", .env$saem.dll)
+  on.exit({assignInMyNamespace(".protectSaemDll", "")}, add=TRUE)
   dopred <- attr(fit, "dopred")
   resMat = fit$resMat
   ares = resMat[, 1]
